<style>
.header_text{
font-size: 30px !important;
text-align: left !important;
}

.header_image_1{
border-right: 2pt solid #302683 !important;
}

.header_image_2{
border-right: 2pt solid #009EE3 !important;
}

.header_image_3{
border-right: 2pt solid #00893E !important;
}

.header_image_4{
border-right: 2pt solid #94C11E !important;
}

.header_image_5{
border-right: 2pt solid #FDC400 !important;
}

.header_image_6{
border-right: 2pt solid #E84D0E !important;
}

.header_image_7{
border-right: 2pt solid #CF0272 !important;
}

.header_image_8{
border-right: 2pt solid #F199C1 !important;
}

<style>

