#from .data import *
#from novainstrumentation.waves import *
#from .figtools import *
from .filter import *
from .freq_analysis import *
#from .niplot import *
#from .peakdelta import *
#from .peaks import *
#from .smooth import *
#from .stat_analysis import *
#from .tictac import *
#from .tools import *
#from .panthomkins import *
